<?xml version="1.0" encoding="UTF-8"?>
<tileset version="1.2" tiledversion="1.2.4" name="grass" tilewidth="128" tileheight="128" tilecount="19" columns="0">
 <grid orientation="orthogonal" width="1" height="1"/>
 <tile id="0">
  <image width="128" height="128" source="../images/tiles/grass.png"/>
 </tile>
 <tile id="1">
  <image width="128" height="128" source="../images/tiles/grass_sprout.png"/>
 </tile>
 <tile id="2">
  <image width="128" height="128" source="../images/tiles/grassCenter.png"/>
 </tile>
 <tile id="3">
  <image width="128" height="128" source="../images/tiles/grassCenter_round.png"/>
 </tile>
 <tile id="4">
  <image width="128" height="128" source="../images/tiles/grassCliff_left.png"/>
 </tile>
 <tile id="5">
  <image width="128" height="128" source="../images/tiles/grassCliff_right.png"/>
 </tile>
 <tile id="6">
  <image width="128" height="128" source="../images/tiles/grassCliffAlt_left.png"/>
 </tile>
 <tile id="7">
  <image width="128" height="128" source="../images/tiles/grassCliffAlt_right.png"/>
 </tile>
 <tile id="8">
  <image width="128" height="128" source="../images/tiles/grassCorner_left.png"/>
 </tile>
 <tile id="9">
  <image width="128" height="128" source="../images/tiles/grassCorner_right.png"/>
 </tile>
 <tile id="10">
  <image width="128" height="128" source="../images/tiles/grassHalf.png"/>
 </tile>
 <tile id="11">
  <image width="128" height="128" source="../images/tiles/grassHalf_left.png"/>
 </tile>
 <tile id="12">
  <image width="128" height="128" source="../images/tiles/grassHalf_mid.png"/>
 </tile>
 <tile id="13">
  <image width="128" height="128" source="../images/tiles/grassHalf_right.png"/>
 </tile>
 <tile id="14">
  <image width="128" height="128" source="../images/tiles/grassHill_left.png"/>
  <objectgroup draworder="index">
   <object id="1" x="0" y="0">
    <polygon points="0,0 128,128 0,128"/>
   </object>
  </objectgroup>
 </tile>
 <tile id="15">
  <image width="128" height="128" source="../images/tiles/grassHill_right.png"/>
  <objectgroup draworder="index">
   <object id="1" x="0" y="128">
    <polygon points="0,0 128,-128 128,0"/>
   </object>
  </objectgroup>
 </tile>
 <tile id="16">
  <image width="128" height="128" source="../images/tiles/grassLeft.png"/>
  <objectgroup draworder="index">
   <object id="4" x="0" y="0" width="128" height="128"/>
  </objectgroup>
 </tile>
 <tile id="17">
  <image width="128" height="128" source="../images/tiles/grassMid.png"/>
 </tile>
 <tile id="18">
  <image width="128" height="128" source="../images/tiles/grassRight.png"/>
 </tile>
</tileset>
