<?xml version="1.0" encoding="UTF-8"?>
<tileset name="standard_tileset" tilewidth="128" tileheight="128" tilecount="11" columns="0">
 <grid orientation="orthogonal" width="1" height="1"/>
 <tile id="0">
  <image width="128" height="128" source="../images/tiles/boxCrate_double.png"/>
 </tile>
 <tile id="1">
  <image width="128" height="128" source="../images/items/coinGold.png"/>
 </tile>
 <tile id="2">
  <image width="128" height="128" source="../images/tiles/grassCenter.png"/>
 </tile>
 <tile id="3">
  <image width="128" height="128" source="../images/tiles/grassCorner_left.png"/>
 </tile>
 <tile id="4">
  <image width="128" height="128" source="../images/tiles/grassCorner_right.png"/>
 </tile>
 <tile id="5">
  <image width="128" height="128" source="../images/tiles/grassHill_left.png"/>
 </tile>
 <tile id="6">
  <image width="128" height="128" source="../images/tiles/grassHill_right.png"/>
 </tile>
 <tile id="7">
  <image width="128" height="128" source="../images/tiles/grassLeft.png"/>
 </tile>
 <tile id="8">
  <image width="128" height="128" source="../images/tiles/grassMid.png"/>
 </tile>
 <tile id="9">
  <image width="128" height="128" source="../images/tiles/grassRight.png"/>
 </tile>
 <tile id="10">
  <image width="128" height="73" source="../images/tiles/stoneHalf.png"/>
  <objectgroup draworder="index">
   <object id="1" x="0.363636" y="12.9091">
    <polygon points="0,0 10.3636,-12.7273 117.818,-12.3636 127.091,0.363636 127.091,48.3636 116.909,58.9091 9.45455,59.4545 0,47.0909"/>
   </object>
  </objectgroup>
 </tile>
</tileset>
