<?xml version="1.0" encoding="UTF-8"?>
<tileset version="1.2" tiledversion="1.2.4" name="dirt" tilewidth="128" tileheight="128" tilecount="18" columns="0">
 <grid orientation="orthogonal" width="1" height="1"/>
 <tile id="0">
  <image width="128" height="128" source="../images/tiles/dirt.png"/>
 </tile>
 <tile id="1">
  <image width="128" height="128" source="../images/tiles/dirtCenter.png"/>
 </tile>
 <tile id="2">
  <image width="128" height="128" source="../images/tiles/dirtCenter_rounded.png"/>
 </tile>
 <tile id="3">
  <image width="128" height="128" source="../images/tiles/dirtCliff_left.png"/>
 </tile>
 <tile id="4">
  <image width="128" height="128" source="../images/tiles/dirtCliff_right.png"/>
 </tile>
 <tile id="5">
  <image width="128" height="128" source="../images/tiles/dirtCliffAlt_left.png"/>
 </tile>
 <tile id="6">
  <image width="128" height="128" source="../images/tiles/dirtCliffAlt_right.png"/>
 </tile>
 <tile id="7">
  <image width="128" height="128" source="../images/tiles/dirtCorner_left.png"/>
 </tile>
 <tile id="8">
  <image width="128" height="128" source="../images/tiles/dirtCorner_right.png"/>
 </tile>
 <tile id="9">
  <image width="128" height="128" source="../images/tiles/dirtHalf.png"/>
  <objectgroup draworder="index">
   <object id="1" x="0" y="12">
    <polygon points="0,0 10.7273,-11.6364 118.182,-11.8182 127.636,-1.81818 127.636,50.1818 116.727,61.4545 10.9091,61.6364 0.181818,49.6364"/>
   </object>
  </objectgroup>
 </tile>
 <tile id="10">
  <image width="128" height="128" source="../images/tiles/dirtHalf_left.png"/>
  <objectgroup draworder="index">
   <object id="2" x="0" y="0">
    <polygon points="0,0 128,0 128,74 0,74"/>
   </object>
  </objectgroup>
 </tile>
 <tile id="11">
  <image width="128" height="128" source="../images/tiles/dirtHalf_mid.png"/>
  <objectgroup draworder="index">
   <object id="1" x="0" y="0" width="128" height="74"/>
  </objectgroup>
 </tile>
 <tile id="12">
  <image width="128" height="128" source="../images/tiles/dirtHalf_right.png"/>
  <objectgroup draworder="index">
   <object id="3" x="0" y="74">
    <polygon points="0,0 128,0 128,-74 0,-74"/>
   </object>
  </objectgroup>
 </tile>
 <tile id="13">
  <image width="128" height="128" source="../images/tiles/dirtHill_left.png"/>
 </tile>
 <tile id="14">
  <image width="128" height="128" source="../images/tiles/dirtHill_right.png"/>
 </tile>
 <tile id="15">
  <image width="128" height="128" source="../images/tiles/dirtLeft.png"/>
 </tile>
 <tile id="16">
  <image width="128" height="128" source="../images/tiles/dirtMid.png"/>
 </tile>
 <tile id="17">
  <image width="128" height="128" source="../images/tiles/dirtRight.png"/>
 </tile>
</tileset>
